@@ -32,12 +32,12 @@
 #include <inttypes.h>
 #include <sys/stat.h>
 
-#include "xine_internal.h"
-#include "xineutils.h"
-#include "audio_out.h"
+#include <xine/xine_internal.h>
+#include <xine/xineutils.h>
+#include <xine/audio_out.h>
 #include "bswap.h"
 
-#define AO_OUT_FILE_IFACE_VERSION 8
+#define AO_OUT_FILE_IFACE_VERSION 9
 
 #define GAP_TOLERANCE        INT_MAX
 
@@ -85,7 +85,7 @@ typedef struct file_driver_s {
 	uint32_t       bits_per_sample;
 	uint32_t       bytes_per_frame;
 
-	char	      *fname;
+	const char    *fname;
 	int            fd;
 	size_t         bytes_written;
 	struct timeval endtime;
@@ -274,20 +274,10 @@ static void ao_file_close(ao_driver_t *this_gen)
 		 this->fname, this->bytes_written / 1024);
 
 	if (lseek(this->fd, 40, SEEK_SET) != -1) {
-		if (write(this->fd, &len, 4) != 4) {
-			xprintf (this->xine, XINE_VERBOSITY_LOG, "audio_file_out: Failed to write header to file '%s': %s\n",
-                                 this->fname, strerror(errno));
-		}
-
+		write(this->fd, &len, 4);
 		len = le2me_32(this->bytes_written + 0x24);
-		if (lseek(this->fd, 4, SEEK_SET) != -1) {
-			if (write(this->fd, &len, 4) != 4) {
-				xprintf (this->xine, XINE_VERBOSITY_LOG,
-                                         "audio_file_out: Failed to write header to file '%s': %s\n",
-                                         this->fname, strerror(errno));
-                        }
-		}
-
+		if (lseek(this->fd, 4, SEEK_SET) != -1)
+			write(this->fd, &len, 4);
 	}
 
 	close(this->fd);
@@ -376,22 +366,6 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen,
 /*
  * class functions
  */
-
-static char* get_identifier (audio_driver_class_t *this_gen) {
-	return "file";
-}
-
-static char* get_description (audio_driver_class_t *this_gen) {
-	return _("xine file audio output plugin");
-}
-
-static void dispose_class (audio_driver_class_t *this_gen) {
-
-	file_class_t *this = (file_class_t *) this_gen;
-
-	free (this);
-}
-
 static void *init_class (xine_t *xine, void *data) {
 
 	file_class_t        *this;
@@ -403,9 +377,9 @@ static void *init_class (xine_t *xine, void *data) {
 		return NULL;
 
 	this->driver_class.open_plugin     = open_plugin;
-	this->driver_class.get_identifier  = get_identifier;
-	this->driver_class.get_description = get_description;
-	this->driver_class.dispose         = dispose_class;
+	this->driver_class.identifier      = "file";
+	this->driver_class.description     = N_("xine file audio output plugin");
+	this->driver_class.dispose         = default_audio_driver_class_dispose;
 
 	this->config = xine->config;
 	this->xine   = xine;
