@@ -27,10 +27,12 @@
 
 #ifdef HAVE_FFMPEG_AVUTIL_H
 #  include <avcodec.h>
-#elif defined HAVE_FFMPEG
-#  include <libavcodec/avcodec.h>
 #else
-#  include "../../libffmpeg/libavcodec/avcodec.h"
+#  include <libavcodec/avcodec.h>
+#endif
+
+#if LIBAVCODEC_VERSION_MAJOR > 51
+#define bits_per_sample bits_per_coded_sample
 #endif
 
 typedef struct ff_codec_s {
