@@ -17,7 +17,7 @@
 #define LOG
 */
 
-#include "xineutils.h"
+#include <xine/xineutils.h>
 #include "bswap.h"
 #include "asfheader.h"
 
@@ -63,7 +63,6 @@ struct asf_header_internal_s {
   /* private part */
   int                     number_count;
   uint16_t                numbers[ASF_MAX_NUM_STREAMS];
-  iconv_t                 iconv_cd;
   uint8_t                *bitrate_pointers[ASF_MAX_NUM_STREAMS];
 };
 
@@ -130,8 +129,7 @@ static uint8_t *asf_reader_get_bytes(asf_reader_t *reader, size_t size) {
 
   if ((reader->size - reader->pos) < size)
     return NULL;
-  buffer = malloc(size);
-  if (!buffer)
+  if (! (buffer = malloc(size)) )
     return NULL;
   memcpy(buffer, reader->buffer + reader->pos, size);
   reader->pos += size;
@@ -152,7 +150,7 @@ static char *asf_reader_get_string(asf_reader_t *reader, size_t size, iconv_t cd
   outbuf = scratch;
   outbytesleft = sizeof(scratch);
   reader->pos += size;
-  if (iconv (cd, (ICONV_CONST char **)&inbuf, &inbytesleft, &outbuf, &outbytesleft) != -1) {
+  if (iconv (cd, (ICONV_CONST char **)&inbuf, &inbytesleft, &outbuf, &outbytesleft) != (size_t)-1) {
     return strdup(scratch);
   } else {
     lprintf("iconv error\n");
@@ -241,8 +239,7 @@ static int asf_header_parse_file_properties(asf_header_t *header, uint8_t *buffe
     return 0;
   }
 
-  asf_file = malloc(sizeof(asf_file_t));
-  if (!asf_file) {
+  if (! (asf_file = malloc(sizeof(asf_file_t))) ) {
     lprintf("cannot allocate asf_file_struct\n");
     return 0;
   }
@@ -296,8 +293,7 @@ static int asf_header_parse_stream_properties(asf_header_t *header, uint8_t *buf
   if (buffer_len < 54)
     goto exit_error;
 
-  asf_stream = malloc(sizeof(asf_stream_t));
-  if (!asf_stream)
+  if (! (asf_stream = malloc(sizeof(asf_stream_t))) )
     goto exit_error;
 
   asf_stream->private_data = NULL;
@@ -358,7 +354,7 @@ exit_error:
 static int asf_header_parse_stream_extended_properties(asf_header_t *header, uint8_t *buffer, int buffer_len) {
   asf_reader_t reader;
   uint32_t flags = 0;
-  uint16_t stream_number = 0;
+  uint16_t stream_number;
   int i;
   int stream_id;
   asf_stream_extension_t *asf_stream_extension;
@@ -366,8 +362,7 @@ static int asf_header_parse_stream_extended_properties(asf_header_t *header, uin
   if (buffer_len < 64)
     return 0;
 
-  asf_stream_extension = malloc(sizeof(asf_stream_extension_t));
-  if (!asf_stream_extension)
+  if (! (asf_stream_extension = malloc(sizeof(asf_stream_extension_t))) )
     return 0;
 
   asf_reader_init(&reader, buffer, buffer_len);
@@ -532,7 +527,7 @@ static int asf_header_parse_metadata(asf_header_t *header_pub, uint8_t *buffer,
 
   for (i = 0; i < records_count; i++)
   {
-    uint16_t index, stream = 0, name_len = 0, data_type;
+    uint16_t index, stream, name_len = 0, data_type;
     uint32_t data_len = 0;
     int stream_id;
 
@@ -637,6 +632,7 @@ static int asf_header_parse_content_description(asf_header_t *header_pub, uint8_
   asf_reader_t reader;
   asf_content_t *content;
   uint16_t title_length, author_length, copyright_length, description_length, rating_length;
+  iconv_t iconv_cd;
 
   if (buffer_len < 10)
     return 0;
@@ -645,6 +641,9 @@ static int asf_header_parse_content_description(asf_header_t *header_pub, uint8_
   if (!content)
     return 0;
 
+  if ( (iconv_cd = iconv_open("UTF-8", "UCS-2LE")) == (iconv_t)-1 )
+    return 0;
+
   asf_reader_init(&reader, buffer, buffer_len);
   asf_reader_get_16(&reader, &title_length);
   asf_reader_get_16(&reader, &author_length);
@@ -652,11 +651,11 @@ static int asf_header_parse_content_description(asf_header_t *header_pub, uint8_
   asf_reader_get_16(&reader, &description_length);
   asf_reader_get_16(&reader, &rating_length);
 
-  content->title = asf_reader_get_string(&reader, title_length, header->iconv_cd);
-  content->author = asf_reader_get_string(&reader, author_length, header->iconv_cd);
-  content->copyright = asf_reader_get_string(&reader, copyright_length, header->iconv_cd);
-  content->description = asf_reader_get_string(&reader, description_length, header->iconv_cd);
-  content->rating = asf_reader_get_string(&reader, rating_length, header->iconv_cd);
+  content->title = asf_reader_get_string(&reader, title_length, iconv_cd);
+  content->author = asf_reader_get_string(&reader, author_length, iconv_cd);
+  content->copyright = asf_reader_get_string(&reader, copyright_length, iconv_cd);
+  content->description = asf_reader_get_string(&reader, description_length, iconv_cd);
+  content->rating = asf_reader_get_string(&reader, rating_length, iconv_cd);
 
   lprintf("title: %d chars: \"%s\"\n", title_length, content->title);
   lprintf("author: %d chars: \"%s\"\n", author_length, content->author);
@@ -665,6 +664,8 @@ static int asf_header_parse_content_description(asf_header_t *header_pub, uint8_
   lprintf("rating: %d chars: \"%s\"\n", rating_length, content->rating);
 
   header->pub.content = content;
+
+  iconv_close(iconv_cd);
   return 1;
 }
 
@@ -683,16 +684,11 @@ asf_header_t *asf_header_new (uint8_t *buffer, int buffer_len) {
   lprintf("parsing_asf_header\n");
   if (buffer_len < 6) {
     printf("invalid buffer size\n");
-    free(asf_header);
     return NULL;
   }
 
-  asf_header->iconv_cd = iconv_open ("UTF-8", "UCS-2LE");
-  if (asf_header->iconv_cd == (iconv_t)-1) {
-    printf("iconv open error\n");
-    free(asf_header);
+  if (! (asf_header = calloc(1, sizeof(asf_header_internal_t))) )
     return NULL;
-  }
 
   asf_reader_init(&reader, buffer, buffer_len);
   asf_reader_get_32(&reader, &object_count);
@@ -829,9 +825,6 @@ void asf_header_delete (asf_header_t *header_pub) {
       asf_header_delete_stream_extended_properties(header->pub.stream_extensions[i]);
   }
   
-  if (header->iconv_cd != (iconv_t)-1)
-    iconv_close (header->iconv_cd);
-
   free(header);
 }
 
