@@ -33,10 +33,10 @@
 #include <stdlib.h>
 #include <ctype.h>
 
-#include "xine_internal.h"
-#include "xineutils.h"
-#include "demux.h"
-#include "buffer.h"
+#include <xine/xine_internal.h>
+#include <xine/xineutils.h>
+#include <xine/demux.h>
+#include <xine/buffer.h>
 #include "bswap.h"
 #include "group_audio.h"
 
@@ -336,12 +336,6 @@ static int demux_aiff_seek (demux_plugin_t *this_gen,
   return this->status;
 }
 
-static void demux_aiff_dispose (demux_plugin_t *this_gen) {
-  demux_aiff_t *this = (demux_aiff_t *) this_gen;
-
-  free(this);
-}
-
 static int demux_aiff_get_status (demux_plugin_t *this_gen) {
   demux_aiff_t *this = (demux_aiff_t *) this_gen;
 
@@ -376,7 +370,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   this->demux_plugin.send_headers      = demux_aiff_send_headers;
   this->demux_plugin.send_chunk        = demux_aiff_send_chunk;
   this->demux_plugin.seek              = demux_aiff_seek;
-  this->demux_plugin.dispose           = demux_aiff_dispose;
+  this->demux_plugin.dispose           = default_demux_plugin_dispose;
   this->demux_plugin.get_status        = demux_aiff_get_status;
   this->demux_plugin.get_stream_length = demux_aiff_get_stream_length;
   this->demux_plugin.get_capabilities  = demux_aiff_get_capabilities;
@@ -387,19 +381,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   switch (stream->content_detection_method) {
 
-  case METHOD_BY_EXTENSION: {
-    const char *extensions, *mrl;
-
-    mrl = input->get_mrl (input);
-    extensions = class_gen->get_extensions (class_gen);
-
-    if (!_x_demux_check_extension (mrl, extensions)) {
-      free (this);
-      return NULL;
-    }
-  }
-  /* falling through is intended */
-
+  case METHOD_BY_MRL:
   case METHOD_BY_CONTENT:
   case METHOD_EXPLICIT:
 
@@ -418,41 +400,20 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 }
 
-static const char *get_description (demux_class_t *this_gen) {
-  return "AIFF file demux plugin";
-}
-
-static const char *get_identifier (demux_class_t *this_gen) {
-  return "AIFF";
-}
-
-static const char *get_extensions (demux_class_t *this_gen) {
-  return "aif aiff";
-}
-
-static const char *get_mimetypes (demux_class_t *this_gen) {
-  return "audio/x-aiff: aif, aiff: AIFF audio;"
-         "audio/aiff: aif, aiff: AIFF audio;"
-         "audio/x-pn-aiff: aif, aiff: AIFF audio;";
-}
-
-static void class_dispose (demux_class_t *this_gen) {
-  demux_aiff_class_t *this = (demux_aiff_class_t *) this_gen;
-
-  free (this);
-}
-
 void *demux_aiff_init_plugin (xine_t *xine, void *data) {
   demux_aiff_class_t     *this;
 
   this = calloc(1, sizeof(demux_aiff_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
-  this->demux_class.get_description = get_description;
-  this->demux_class.get_identifier  = get_identifier;
-  this->demux_class.get_mimetypes   = get_mimetypes;
-  this->demux_class.get_extensions  = get_extensions;
-  this->demux_class.dispose         = class_dispose;
+  this->demux_class.description     = N_("AIFF file demux plugin");
+  this->demux_class.identifier      = "AIFF";
+  this->demux_class.mimetypes       =
+    "audio/x-aiff: aif, aiff: AIFF audio;"
+    "audio/aiff: aif, aiff: AIFF audio;"
+    "audio/x-pn-aiff: aif, aiff: AIFF audio;";
+  this->demux_class.extensions      = "aif aiff";
+  this->demux_class.dispose         = default_demux_class_dispose;
 
   return this;
 }
