@@ -42,10 +42,10 @@
 #define LOG
 */
 
-#include "xine_internal.h"
-#include "xineutils.h"
-#include "compat.h"
-#include "demux.h"
+#include <xine/xine_internal.h>
+#include <xine/xineutils.h>
+#include <xine/compat.h>
+#include <xine/demux.h>
 #include "bswap.h"
 #include "group_games.h"
 
@@ -141,6 +141,7 @@ static int demux_mve_send_chunk(demux_plugin_t *this_gen) {
   demux_mve_t *this = (demux_mve_t *) this_gen;
 
   buf_element_t *buf = NULL;
+  int64_t text_pts = 0;
   int64_t audio_pts = 0;
   unsigned char preamble[PREAMBLE_SIZE];
   unsigned int chunk_tag;
@@ -263,7 +264,7 @@ static int demux_mve_send_chunk(demux_plugin_t *this_gen) {
       }
       this->video_pts += WC3_PTS_INC;
     } else if (chunk_tag == TEXT_TAG) {
-      /*text_pts = this->video_pts - WC3_PTS_INC;*/
+      text_pts = this->video_pts - WC3_PTS_INC;
 
       /* unhandled thus far */
       this->input->seek(this->input, chunk_size, SEEK_CUR);
@@ -377,15 +378,15 @@ static int open_mve_file(demux_mve_t *this) {
   this->number_of_shots = _X_LE_32(&preamble[0]);
 
   /* allocate space for the shot offset index and set offsets to 0 */
-  this->shot_offsets = calloc(this->number_of_shots, sizeof(off_t));
+  this->shot_offsets = xine_xcalloc(this->number_of_shots, sizeof(off_t));
   this->current_shot = 0;
 
   /* skip the SOND chunk */
   this->input->seek(this->input, 12, SEEK_CUR);
 
   /* load the palette chunks */
-  this->palettes = calloc(this->number_of_shots, PALETTE_SIZE *
-    sizeof(palette_entry_t));
+  this->palettes = xine_xcalloc(this->number_of_shots, PALETTE_SIZE *
+				sizeof(palette_entry_t));
 
   if (!this->shot_offsets || !this->palettes) {
     free (this->shot_offsets);
@@ -689,19 +690,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   switch (stream->content_detection_method) {
 
-  case METHOD_BY_EXTENSION: {
-    const char *extensions, *mrl;
-
-    mrl = input->get_mrl (input);
-    extensions = class_gen->get_extensions (class_gen);
-
-    if (!_x_demux_check_extension (mrl, extensions)) {
-      free (this);
-      return NULL;
-    }
-  }
-  /* falling through is intended */
-
+  case METHOD_BY_MRL:
   case METHOD_BY_CONTENT:
   case METHOD_EXPLICIT:
 
@@ -720,39 +709,17 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 }
 
-static const char *get_description (demux_class_t *this_gen) {
-  return "Wing Commander III Movie (MVE) demux plugin";
-}
-
-static const char *get_identifier (demux_class_t *this_gen) {
-  return "WC3 Movie";
-}
-
-static const char *get_extensions (demux_class_t *this_gen) {
-  return "mve";
-}
-
-static const char *get_mimetypes (demux_class_t *this_gen) {
-  return NULL;
-}
-
-static void class_dispose (demux_class_t *this_gen) {
-  demux_mve_class_t *this = (demux_mve_class_t *) this_gen;
-
-  free (this);
-}
-
 void *demux_wc3movie_init_plugin (xine_t *xine, void *data) {
   demux_mve_class_t     *this;
 
   this = calloc(1, sizeof(demux_mve_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
-  this->demux_class.get_description = get_description;
-  this->demux_class.get_identifier  = get_identifier;
-  this->demux_class.get_mimetypes   = get_mimetypes;
-  this->demux_class.get_extensions  = get_extensions;
-  this->demux_class.dispose         = class_dispose;
+  this->demux_class.description     = N_("Wing Commander III Movie (MVE) demux plugin");
+  this->demux_class.identifier      = "WC3 Movie";
+  this->demux_class.mimetypes       = NULL;
+  this->demux_class.extensions      = "mve";
+  this->demux_class.dispose         = default_demux_class_dispose;
 
   return this;
 }
