@@ -36,9 +36,9 @@
 #define LOG
 */
 
-#include "xine_internal.h"
-#include "xineutils.h"
-#include "input_plugin.h"
+#include <xine/xine_internal.h>
+#include <xine/xineutils.h>
+#include <xine/input_plugin.h>
 #include "net_buf_ctrl.h"
 
 #define BUFSIZE                 1024
@@ -79,9 +79,10 @@ static off_t stdin_plugin_get_current_pos (input_plugin_t *this_gen);
 
 
 static off_t stdin_plugin_read (input_plugin_t *this_gen,
-				char *buf, off_t len) {
+				void *buf_gen, off_t len) {
 
   stdin_input_plugin_t  *this = (stdin_input_plugin_t *) this_gen;
+  char *buf = (char *)buf_gen;
   off_t n, total;
 
   lprintf ("reading %"PRId64" bytes...\n", len);
@@ -354,21 +355,6 @@ static input_plugin_t *stdin_class_get_instance (input_class_t *class_gen,
 /*
  * stdin input plugin class stuff
  */
-
-static const char *stdin_class_get_description (input_class_t *this_gen) {
-  return _("stdin streaming input plugin");
-}
-
-static const char *stdin_class_get_identifier (input_class_t *this_gen) {
-  return "stdin_fifo";
-}
-
-static void stdin_class_dispose (input_class_t *this_gen) {
-  stdin_input_class_t  *this = (stdin_input_class_t *) this_gen;
-
-  free (this);
-}
-
 static void *init_class (xine_t *xine, void *data) {
 
   stdin_input_class_t  *this;
@@ -378,11 +364,11 @@ static void *init_class (xine_t *xine, void *data) {
   this->xine   = xine;
 
   this->input_class.get_instance       = stdin_class_get_instance;
-  this->input_class.get_identifier     = stdin_class_get_identifier;
-  this->input_class.get_description    = stdin_class_get_description;
+  this->input_class.identifier         = "stdin_fifo";
+  this->input_class.description        = N_("stdin streaming input plugin");
   this->input_class.get_dir            = NULL;
   this->input_class.get_autoplay_list  = NULL;
-  this->input_class.dispose            = stdin_class_dispose;
+  this->input_class.dispose            = default_input_class_dispose;
   this->input_class.eject_media        = NULL;
 
   return this;
@@ -394,6 +380,6 @@ static void *init_class (xine_t *xine, void *data) {
 
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_INPUT, 17, "stdin", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_INPUT, 18, "stdin", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
