@@ -26,11 +26,11 @@
 
 #include <stdio.h>
 
-#include "xine_internal.h"
-#include "xineutils.h"
-#include "post.h"
+#include <xine/xine_internal.h>
+#include <xine/xineutils.h>
+#include <xine/post.h>
 #include "dsp.h"
-#include "resample.h"
+#include <xine/resample.h>
 
 #include "audio_filters.h"
 
@@ -643,7 +643,7 @@ static post_plugin_t *stretch_open_plugin(post_class_t *class_gen, int inputs,
 
   pthread_mutex_init (&this->lock, NULL);
 
-  set_parameters (&this->post.xine_post, &init_params);
+  set_parameters ((xine_post_t *)&this->post, &init_params);
 
   port = _x_post_intercept_audio_port(&this->post, audio_target[0], &input, &output);
   port->new_port.open       = stretch_port_open;
@@ -663,33 +663,18 @@ static post_plugin_t *stretch_open_plugin(post_class_t *class_gen, int inputs,
   return &this->post;
 }
 
-static char *stretch_get_identifier(post_class_t *class_gen)
-{
-  return "stretch";
-}
-
-static char *stretch_get_description(post_class_t *class_gen)
-{
-  return "Time stretch by a given factor, optionally preserving pitch";
-}
-
-static void stretch_class_dispose(post_class_t *class_gen)
-{
-  free(class_gen);
-}
-
 /* plugin class initialization function */
 void *stretch_init_plugin(xine_t *xine, void *data)
 {
-  post_class_stretch_t *class = (post_class_stretch_t *)malloc(sizeof(post_class_stretch_t));
+  post_class_stretch_t *class = (post_class_stretch_t *)xine_xmalloc(sizeof(post_class_stretch_t));
 
   if (!class)
     return NULL;
 
   class->post_class.open_plugin     = stretch_open_plugin;
-  class->post_class.get_identifier  = stretch_get_identifier;
-  class->post_class.get_description = stretch_get_description;
-  class->post_class.dispose         = stretch_class_dispose;
+  class->post_class.identifier      = "stretch";
+  class->post_class.description     = N_("Time stretch by a given factor, optionally preserving pitch");
+  class->post_class.dispose         = default_post_class_dispose;
 
   class->xine                       = xine;
 
