@@ -35,8 +35,8 @@
 #define LOG
 */
 
-#include "xineutils.h"
-#include "post.h"
+#include <xine/xineutils.h>
+#include <xine/post.h>
 
 #include "audio_filters.h"
 
@@ -315,7 +315,7 @@ static post_plugin_t *upmix_mono_open_plugin(post_class_t *class_gen, int inputs
 
   pthread_mutex_init (&this->lock, NULL);
 
-  set_parameters (&this->post.xine_post, &init_params);
+  set_parameters ((xine_post_t *)&this->post, &init_params);
 
   port = _x_post_intercept_audio_port(&this->post, audio_target[0], &input, &output);
   port->new_port.open       = upmix_mono_port_open;
@@ -333,33 +333,18 @@ static post_plugin_t *upmix_mono_open_plugin(post_class_t *class_gen, int inputs
   return &this->post;
 }
 
-static char *upmix_mono_get_identifier(post_class_t *class_gen)
-{
-  return "upmix_mono";
-}
-
-static char *upmix_mono_get_description(post_class_t *class_gen)
-{
-  return "converts Mono into Stereo";
-}
-
-static void upmix_mono_class_dispose(post_class_t *class_gen)
-{
-  free(class_gen);
-}
-
 /* plugin class initialization function */
 void *upmix_mono_init_plugin(xine_t *xine, void *data)
 {
-  post_class_upmix_mono_t *class = (post_class_upmix_mono_t *)malloc(sizeof(post_class_upmix_mono_t));
+  post_class_upmix_mono_t *class = (post_class_upmix_mono_t *)xine_xmalloc(sizeof(post_class_upmix_mono_t));
 
   if (!class)
     return NULL;
 
   class->post_class.open_plugin     = upmix_mono_open_plugin;
-  class->post_class.get_identifier  = upmix_mono_get_identifier;
-  class->post_class.get_description = upmix_mono_get_description;
-  class->post_class.dispose         = upmix_mono_class_dispose;
+  class->post_class.identifier      = "upmix_mono";
+  class->post_class.description     = N_("converts Mono into Stereo");
+  class->post_class.dispose         = default_post_class_dispose;
 
   class->xine                       = xine;
 
