@@ -40,9 +40,9 @@
 #define METRONOM_INTERNAL
 #define METRONOM_CLOCK_INTERNAL
 
-#include "xine_internal.h"
-#include "metronom.h"
-#include "xineutils.h"
+#include <xine/xine_internal.h>
+#include <xine/metronom.h>
+#include <xine/xineutils.h>
 
 #define MAX_AUDIO_DELTA        1600
 #define AUDIO_SAMPLE_NUM      32768
@@ -745,41 +745,23 @@ static void metronom_clock_set_option (metronom_clock_t *this,
 
 static int64_t metronom_get_option (metronom_t *this, int option) {
 
-  int64_t result;
-
-  pthread_mutex_lock (&this->lock);
-
-  if (this->master) {
-    result = this->master->get_option(this->master, option);
-    pthread_mutex_unlock (&this->lock);
-    return result;
-  }
+  if (this->master)
+    return this->master->get_option(this->master, option);
 
   switch (option) {
   case METRONOM_AV_OFFSET:
-    result = this->av_offset;
-    break;
+    return this->av_offset;
   case METRONOM_SPU_OFFSET:
-    result = this->spu_offset;
-    break;
+    return this->spu_offset;
   case METRONOM_FRAME_DURATION:
-    result = this->img_duration;
-    break;
+    return this->img_duration;
   case METRONOM_VPTS_OFFSET:
-    result = this->vpts_offset;
-    break;
+    return this->vpts_offset;
   case METRONOM_PREBUFFER:
-    result = this->prebuffer;
-    break;
-  default:
-    result = 0;
-    xprintf(this->xine, XINE_VERBOSITY_NONE, "unknown option in get_option: %d\n", option);
-    break;
+    return this->prebuffer;
   }
-
-  pthread_mutex_unlock (&this->lock);
-
-  return result;
+  xprintf(this->xine, XINE_VERBOSITY_NONE, "unknown option in get_option: %d\n", option);
+  return 0;
 }
 
 static int64_t metronom_clock_get_option (metronom_clock_t *this, int option) {
@@ -866,7 +848,8 @@ static void metronom_unregister_scr (metronom_clock_t *this, scr_plugin_t *scr)
   this->scr_master = get_master_scr(this);
 }
 
-static int metronom_sync_loop (metronom_clock_t *this) {
+static void *metronom_sync_loop (void *const this_gen) {
+  metronom_clock_t *const this = (metronom_clock_t *const)this_gen;
 
   struct timeval tv;
   struct timespec ts;
@@ -889,7 +872,7 @@ static int metronom_sync_loop (metronom_clock_t *this) {
 
     pthread_mutex_unlock (&this->lock);
   }
-  return 0;
+  return NULL;
 }
 
 static void metronom_exit (metronom_t *this) {
@@ -1005,7 +988,7 @@ metronom_clock_t *_x_metronom_clock_init(xine_t *xine)
   this->thread_running       = 1;
 
   if ((err = pthread_create(&this->sync_thread, NULL,
-			    (void*(*)(void*)) metronom_sync_loop, this)) != 0)
+			    metronom_sync_loop, this)) != 0)
     xprintf(this->xine, XINE_VERBOSITY_NONE, "cannot create sync thread (%s)\n",
 	    strerror(err));
 
